
#include "common.hxx"
#include "basic.hxx"
#include "Config.hxx"
#include "Address.hxx"
#include "Input.hxx"
#include "Output.hxx"
#include "LongUDPSocket.hxx"
#include "ObjectPool.hxx"
#include "objects.hxx"
using namespace Rescue;

// by koto
int fread2(S32* buffer, int size, int count, FILE* fp) {
	ASSERT(size == sizeof(S32) && count == 1);
	
	Byte b1;
	if(fread(&b1, sizeof(Byte), 1, fp) != 1)
		return 0;
	Byte b2;
	if(fread(&b2, sizeof(Byte), 1, fp) != 1)
		return 0;
	Byte b3;
	if(fread(&b3, sizeof(Byte), 1, fp) != 1)
		return 0;
	Byte b4;
	if(fread(&b4, sizeof(Byte), 1, fp) != 1)
		return 0;
	
	*buffer = (S32)b1 | ((S32)b2 << 8) | ((S32)b3 << 16) | ((S32)b4 << 24);
	return 1;
}
//

// by Hatayama
int DetPositions( std::vector<S32> * pEntities, S32 nRecCount, S32 nAllRecCount) {
	S32		loop1, loop2, nCntRec, tmp;
	bool	bFlag;

	srand( (unsigned)time( NULL ) );
	pEntities->clear();

	nCntRec = 0;
	for ( loop1=0; loop1<nRecCount; loop1++ ){
		tmp = (S32)rand() % nAllRecCount;
		bFlag = false;
		for (loop2=0; loop2<nCntRec; loop2++){
			if (tmp == (*pEntities)[(int)loop2] )
				bFlag = true;
		}
		if (bFlag != true){
			pEntities->push_back(tmp);
			nCntRec++;
		}
	}

	return 1;
}
//

int main2(int argc, char** argv);
void makeWorld(ObjectPool& pool, int number);

int main(int argc, char** argv)
{
	try {
		return main2(argc, argv);
	} catch(std::exception& e) {
		fprintf(stderr, "%s\n", (const char*)e.what());
	} catch(...) {
		fprintf(stderr, "error...\n");
	}
	return 1;
}

int main2(int argc, char** argv)
{
	const char* host = "localhost";
	Config config;
	int number = 50;

	// 
	int i;
	for(i=1; i<argc; i++) {
		if(argv[i][0] == '-') {
			if(strcmp(argv[i], "-file") == 0) {
				if(++i < argc) {
					config.readConfigFile(argv[i]);
				} else {
					fprintf(stderr, "error: '-file'\n");
				}
			} else if(strcmp(argv[i], "-n") == 0) {
				if(++i < argc) {
					number = atoi(argv[i]);
				} else {
					fprintf(stderr, "error: '-file'\n");
				}
			} else {
				if(++i < argc) {
					if(!config.setValue(argv[i-1]+1, argv[i]))
						fprintf(stderr, "error: '%s'\n", argv[i]);
				} else {
					fprintf(stderr, "error: '%s'\n", argv[i-1]);
				}
			}
		} else {
			fprintf(stderr, "error: '%s'\n", argv[i]);
		}
	}

	// \Pbg쐬
	Address address = Address(INADDR_ANY, config.gis_port(), true);
	LongUDPSocket socket(address, config.textlogname_gis());
	Input input;
	Output output;
	output.setWait(config.send_udp_wait());
	Address to(host, config.port());	//M

	printf("start\n");

	// bZ[W[v
	S32 time = 0;
	ObjectPool pool;
	makeWorld(pool, number);
	for(;;) {
		try {
			socket.receive(input);	// TODO: G[OŕԂ	//f[^MAinput ֒~B
			for(;;) {
				Header header = input.get();	//input (4oCg)f[^oAheader ֑
				if(header == HEADER_NULL)
					break;

				S32 size = input.get();
				Input::Cursor start = input.cursor();	//ݓǂݏoĂʒuL

				switch(header) {
				default:
					printf("Unknown header. %lx\n", (long)header);
					break;
				case KG_CONNECT:
					printf("KG_CONNECT\n");
					to = socket.addressRecievedFrom();
					output.clear();
					output.put(GK_CONNECT_OK);
					output.put(~(S32)0);
					pool.output(output, 0);
					output.put(HEADER_NULL);
					socket.send(output, to);
					break;
				case KG_ACKNOWLEDGE:
					printf("KG_ACKNOWLEDGE\n");
					break;
				case KG_UPDATE:
					printf("KG_UPDATE\n");
					time = input.get();
					pool.restructure(time, input);
					break;
				}

				input.setCursor(start);
				input.skip(size);						//size oCgXLbv
			}
		} catch(std::exception& e) {
			fprintf(stderr, "%s\n", (const char*)e.what());
		} catch(...) {
			fprintf(stderr, "error...\n");
		}
	}
	
	return 0;
}

// GIS by M.Hatayama
// modified by M.H 2000.8.24
void makeWorld(ObjectPool& pool, int number)
{
	FILE *		fpTxt;
	FILE *		fpRoad;
	FILE *		fpNode;
	FILE *		fpBuilding;
	int			result;
	S32			nCntRoad, nCntNode, nCntAllBuilding, nCntBuilding;
	S32			nCntAmbulanceCenter, nCntPoliceOffice, nCntFireStation, nCntRefuge;
	S32			nCntRec, nCntRec2;
	S32			tmp, loop1, loop2, tmp_byte;
	S32			cntBA, cntBP, cntBF, cntBR, cntB;

	World *		pWorld;
	Road *		pRoad;
	Node *		pNode;
	Building *	pAllBuilding;
	Building *	pBuilding;
	Building *	pAmbulanceCenter;
	Building *	pPoliceOffice;
	Building *	pFireStation;
	Building *	pRefuge;

	Bytes		bytes;

	const int size = 1024;
	char buffer[size];
	
	bool		bIniFileF;
	int			AllListOutput;
	char		FirePoint_FN[256];
	char		AmbulanceCenter_FN[256];
	char		FireStation_FN[256];
	char		PoliceOffice_FN[256];
	char		Refuge_FN[256];
	char		CivilianPoint_FN[256];
	char		AmbulanceTeamPoint_FN[256];
	char		FireBrigadePoint_FN[256];
	char		PoliceForcePoint_FN[256];
	
	//Temp
	nCntAmbulanceCenter = nCntPoliceOffice = nCntFireStation = 1;
	nCntRefuge = 4;

	//load initial data
	char name1[size];
	char name2[size];

	if( (fpTxt = fopen("gisini.txt", "r+t") ) == NULL){
		bIniFileF = false;
		AllListOutput = 0;
	}
	else {
		bIniFileF = true;
		while(fgets(buffer, size, fpTxt) != NULL) {
			if (buffer[0] == '#' || buffer[0] == '\n')	continue;

			char *p = buffer;
			while(*p != '\0') {
				if(*p == ':') *p = ' ';
				if(*p == '=') *p = ' ';
				p++;
			}
			int n = sscanf(buffer, "%s %s", name1, name2);
			if(n == 2) {
				if ( strcmp( name1, "AllListOutput") == 0 )
					AllListOutput = atoi(name2);

				if ( strcmp( name1, "FirePoint") == 0 )
					sprintf( FirePoint_FN, "%s", name2 );

				if ( strcmp( name1, "AmbulanceCenter") == 0 )
					sprintf( AmbulanceCenter_FN, "%s", name2 );

				if ( strcmp( name1, "FireStation") == 0 )
					sprintf( FireStation_FN, "%s", name2 );

				if ( strcmp( name1, "PoliceOffice") == 0 )
					sprintf( PoliceOffice_FN, "%s", name2 );

				if ( strcmp( name1, "Refuge") == 0 )
					sprintf( Refuge_FN, "%s", name2 );

				if ( strcmp( name1, "CivilianPoint") == 0 )
					sprintf( CivilianPoint_FN, "%s", name2 );

				if ( strcmp( name1, "AmbulanceTeamPoint") == 0 )
					sprintf( AmbulanceTeamPoint_FN, "%s", name2 );

				if ( strcmp( name1, "FireBrigadePoint") == 0 )
					sprintf( FireBrigadePoint_FN, "%s", name2 );

				if ( strcmp( name1, "PoliceForcePoint") == 0 )
					sprintf( PoliceForcePoint_FN, "%s", name2 );
			}
		}
		fclose(fpTxt);
	}

	//world data
	pWorld = new World;
	pool.testAdd( pWorld );

	//load road data
	if( (fpRoad = fopen("road.bin", "r+b") ) == NULL){
		printf("cannot find file --- road.bin\n");
		return;
	}
	result = fseek( fpRoad, 0L, SEEK_SET);
	result = fseek( fpRoad, sizeof( S32 )*3, SEEK_CUR);
	result = fread2( &nCntRoad, sizeof( S32 ), 1, fpRoad );
	pRoad = new Road[nCntRoad];
printf("Road:%ld\n", nCntRoad );

	//load noad data
	if( (fpNode = fopen("node.bin", "r+b") ) == NULL){
		printf("cannot find file --- node.bin\n");
		fclose(fpRoad);
		return;
	}
	result = fseek( fpNode, 0L, SEEK_SET);
	result = fseek( fpNode, sizeof( S32 )*3, SEEK_CUR);
	result = fread2( &nCntNode, sizeof( S32 ), 1, fpNode );
	pNode = new Node[nCntNode];
printf("Node:%ld\n", nCntNode );

	//load building data
	if( (fpBuilding = fopen("building.bin", "r+b") ) == NULL){
		printf("cannot find file --- road.bin\n");
		fclose(fpRoad);
		fclose(fpNode);
		return;
	}
	result = fseek( fpBuilding, 0L, SEEK_SET);
	result = fseek( fpBuilding, sizeof( S32 )*3, SEEK_CUR);
	result = fread2( &nCntAllBuilding, sizeof( S32 ), 1, fpBuilding );
	nCntBuilding = nCntAllBuilding - nCntRefuge - nCntAmbulanceCenter - nCntPoliceOffice - nCntFireStation;
	pBuilding = new Building[nCntBuilding];
printf("Building:%ld\n", nCntBuilding );
	pAmbulanceCenter = new AmbulanceCenter[nCntAmbulanceCenter];
printf("AmbulanceCenter:%ld\n", nCntAmbulanceCenter );
	pPoliceOffice = new PoliceOffice[nCntPoliceOffice];
printf("PoliceOffice:%ld\n", nCntPoliceOffice );
	pFireStation = new FireStation[nCntFireStation];
printf("FireStation:%ld\n", nCntFireStation );
	pRefuge = new Refuge[nCntRefuge];
printf("Refuge:%ld\n", nCntRefuge );

/*
	for(int i=0 ;i<4; i++) {
		Refuge* refuge = (Refuge*)&pBuilding[nCntBuilding/4*i];
		new(refuge) Refuge();	// BAD!BAD!BAD!BAD!
	}
*/
	//make road data
	for ( loop1=0; loop1<nCntRoad; loop1++ ){
		result = fread2( &nCntRec, sizeof( S32 ), 1, fpRoad );
		result = fread2( &tmp, sizeof( S32 ), 1, fpRoad );
		pRoad[loop1].setId( tmp );
		result = fread2( &tmp, sizeof( S32 ), 1, fpRoad );
		pNode[tmp-1].testConnect(0, &pRoad[loop1], true);
		result = fread2( &tmp, sizeof( S32 ), 1, fpRoad );
		pNode[tmp-1].testConnect(0, &pRoad[loop1], false);
		result = fread2( &tmp, sizeof( S32 ), 1, fpRoad );
		pRoad[loop1].setLength(0, tmp);
		result = fread2( &tmp, sizeof( S32 ), 1, fpRoad );
		pRoad[loop1].setRoadKind(0, tmp);
		result = fread2( &tmp, sizeof( S32 ), 1, fpRoad );
		pRoad[loop1].setCarsPassToHead(0, tmp);
		result = fread2( &tmp, sizeof( S32 ), 1, fpRoad );
		pRoad[loop1].setCarsPassToTail(0, tmp);
		result = fread2( &tmp, sizeof( S32 ), 1, fpRoad );
		pRoad[loop1].setHumansPassToHead(0, tmp);
		result = fread2( &tmp, sizeof( S32 ), 1, fpRoad );
		pRoad[loop1].setHumansPassToTail(0, tmp);
		result = fread2( &tmp, sizeof( S32 ), 1, fpRoad );
		pRoad[loop1].setWidth(0, tmp);
		result = fread2( &tmp, sizeof( S32 ), 1, fpRoad );
		pRoad[loop1].setBlock(0, tmp);
		result = fread2( &tmp, sizeof( S32 ), 1, fpRoad );
		pRoad[loop1].setRepairCost(0, tmp);
		result = fread2( &tmp, sizeof( S32 ), 1, fpRoad );
		pRoad[loop1].setMedianStrip(0, tmp);
		result = fread2( &tmp, sizeof( S32 ), 1, fpRoad );
		pRoad[loop1].setLinesToHead(0, tmp);
		result = fread2( &tmp, sizeof( S32 ), 1, fpRoad );
		pRoad[loop1].setLinesToTail(0, tmp);
		result = fread2( &tmp, sizeof( S32 ), 1, fpRoad );
		pRoad[loop1].setWidthForWalkers(0, tmp);

		pool.testAdd( &pRoad[loop1] );
	}
	fclose(fpRoad);

	//make node data
	for ( loop1=0; loop1<nCntNode; loop1++ ){
		result = fread2( &nCntRec, sizeof( S32 ), 1, fpNode );
		result = fread2( &tmp, sizeof( S32 ), 1, fpNode );
		pNode[loop1].setId( tmp );
		result = fread2( &tmp, sizeof( S32 ), 1, fpNode );
		pNode[loop1].setX(0, tmp);
		result = fread2( &tmp, sizeof( S32 ), 1, fpNode );
		pNode[loop1].setY(0, tmp);

		result = fread2( &nCntRec2, sizeof( S32 ), 1, fpNode );
		for (loop2=0; loop2<nCntRec2; loop2++){
			result = fread2( &tmp, sizeof( S32 ), 1, fpNode );
			//	eIDs & bIDs
		}

		result = fread2( &tmp, sizeof( S32 ), 1, fpNode );
		pNode[loop1].setSignal(0, tmp);

		bytes.clear();
		for (loop2=0; loop2<nCntRec2; loop2++){
			result = fread2( &tmp, sizeof( S32 ), 1, fpNode );
			tmp_byte = tmp & 0xff000000;
			bytes.push_back( (Byte)(tmp_byte >> 24) );
			tmp_byte = tmp & 0x00ff0000;
			bytes.push_back( (Byte)(tmp_byte >> 16) );
			tmp_byte = tmp & 0x0000ff00;
			bytes.push_back( (Byte)(tmp_byte >> 8) );
			tmp_byte = tmp & 0x000000ff;
			bytes.push_back( (Byte)tmp_byte );
		}
		pNode[loop1].setShortcutToTurn(0, bytes);

		bytes.clear();
		for (loop2=0; loop2<nCntRec2; loop2++){
			result = fread2( &tmp, sizeof( S32 ), 1, fpNode );
			tmp_byte = tmp & 0xff000000;
			bytes.push_back( (Byte)(tmp_byte >> 24) );
			tmp_byte = tmp & 0x00ff0000;
			bytes.push_back( (Byte)(tmp_byte >> 16) );
			tmp_byte = tmp & 0x0000ff00;
			bytes.push_back( (Byte)(tmp_byte >> 8) );
			tmp_byte = tmp & 0x000000ff;
			bytes.push_back( (Byte)tmp_byte );

			result = fread2( &tmp, sizeof( S32 ), 1, fpNode );
			tmp_byte = tmp & 0xff000000;
			bytes.push_back( (Byte)(tmp_byte >> 24) );
			tmp_byte = tmp & 0x00ff0000;
			bytes.push_back( (Byte)(tmp_byte >> 16) );
			tmp_byte = tmp & 0x0000ff00;
			bytes.push_back( (Byte)(tmp_byte >> 8) );
			tmp_byte = tmp & 0x000000ff;
			bytes.push_back( (Byte)tmp_byte );
		}
		pNode[loop1].setPocketToTurnAcross(0, bytes);

		bytes.clear();
		for (loop2=0; loop2<nCntRec2; loop2++){
			result = fread2( &tmp, sizeof( S32 ), 1, fpNode );
			tmp_byte = tmp & 0xff000000;
			bytes.push_back( (Byte)(tmp_byte >> 24) );
			tmp_byte = tmp & 0x00ff0000;
			bytes.push_back( (Byte)(tmp_byte >> 16) );
			tmp_byte = tmp & 0x0000ff00;
			bytes.push_back( (Byte)(tmp_byte >> 8) );
			tmp_byte = tmp & 0x000000ff;
			bytes.push_back( (Byte)tmp_byte );

			result = fread2( &tmp, sizeof( S32 ), 1, fpNode );
			tmp_byte = tmp & 0xff000000;
			bytes.push_back( (Byte)(tmp_byte >> 24) );
			tmp_byte = tmp & 0x00ff0000;
			bytes.push_back( (Byte)(tmp_byte >> 16) );
			tmp_byte = tmp & 0x0000ff00;
			bytes.push_back( (Byte)(tmp_byte >> 8) );
			tmp_byte = tmp & 0x000000ff;
			bytes.push_back( (Byte)tmp_byte );

			result = fread2( &tmp, sizeof( S32 ), 1, fpNode );
			tmp_byte = tmp & 0xff000000;
			bytes.push_back( (Byte)(tmp_byte >> 24) );
			tmp_byte = tmp & 0x00ff0000;
			bytes.push_back( (Byte)(tmp_byte >> 16) );
			tmp_byte = tmp & 0x0000ff00;
			bytes.push_back( (Byte)(tmp_byte >> 8) );
			tmp_byte = tmp & 0x000000ff;
			bytes.push_back( (Byte)tmp_byte );
		}
		pNode[loop1].setSignalTiming(0, bytes);

//printf("b:%ld, %ld,%ld\n", pNode[loop1].id(), pNode[loop1].x() ,pNode[loop1].y() );		
		pool.testAdd( &pNode[loop1] );
	}
	fclose(fpNode);

//AmbulanceCenter
//PoliceOffice
//FireStation
//Refuge
	//make building data
	S32					nFirePointCnt;
	std::vector<S32>	FirePointIDs;
	bool				bFire = false;

	nFirePointCnt = 5;
	DetPositions(&FirePointIDs, nFirePointCnt, nCntAllBuilding);

	std::vector<S32>	exBuildings;
	DetPositions(&exBuildings, nCntAmbulanceCenter+nCntPoliceOffice+nCntFireStation+nCntRefuge, nCntAllBuilding);

	cntBA = cntBP = cntBF = cntBR = cntB = 0;
	for ( loop1=0; loop1<nCntAllBuilding; loop1++ ){
//printf("%ld/%ld\n", loop1, nCntBuilding);
		if ( loop1 == nCntAllBuilding  - 1 - exBuildings[0] ){
			pAllBuilding = & pAmbulanceCenter[cntBA];
			cntBA = cntBA +1;
		}
		else if (loop1 == nCntAllBuilding  - 1 - exBuildings[1] ){
			pAllBuilding = & pPoliceOffice[cntBP];
			cntBP = cntBP +1;
		}
		else if (loop1 == nCntAllBuilding  - 1 - exBuildings[2] ){
			pAllBuilding = & pFireStation[cntBF];
			cntBF = cntBF +1;
		}
		else if (loop1 == nCntAllBuilding  - 1 - exBuildings[3] || 
				 loop1 == nCntAllBuilding  - 1 - exBuildings[4] ||
				 loop1 == nCntAllBuilding  - 1 - exBuildings[5] || 
				 loop1 == nCntAllBuilding  - 1 - exBuildings[6]){
			pAllBuilding = & pRefuge[cntBR];
			cntBR = cntBR +1;
		}
		else{
			pAllBuilding = & pBuilding[cntB];
			cntB = cntB +1;
		}

		result = fread2( &nCntRec, sizeof( S32 ), 1, fpBuilding );
		result = fread2( &tmp, sizeof( S32 ), 1, fpBuilding );
		pAllBuilding->setId( tmp );
		result = fread2( &tmp, sizeof( S32 ), 1, fpBuilding );
		pAllBuilding->setX(0, tmp);
		result = fread2( &tmp, sizeof( S32 ), 1, fpBuilding );
		pAllBuilding->setY(0, tmp);
		result = fread2( &tmp, sizeof( S32 ), 1, fpBuilding );
		pAllBuilding->setFloors(0, tmp);
		result = fread2( &tmp, sizeof( S32 ), 1, fpBuilding );
		pAllBuilding->setBuildingAttributes(0, tmp);
		result = fread2( &tmp, sizeof( S32 ), 1, fpBuilding );
//		if (bFire != TRUE){
//			tmp = 1;
//			printf("%ld, %ld\n", pBuilding[loop1].x(), pBuilding[loop1].y() );
//			bFire = TRUE;
//		}
/*
		if (pBuilding[loop1].x() == 23046400 && pBuilding[loop1].y() == 3731000 ){
			tmp = 1;
		}
		if (pBuilding[loop1].x() == 22776000 && pBuilding[loop1].y() == 3621000 ){
			tmp = 1;
		}
		if (pBuilding[loop1].x() == 22942100 && pBuilding[loop1].y() == 3796300 ){
			tmp = 1;
		}
		if (pBuilding[loop1].x() == 22975800 && pBuilding[loop1].y() == 3702000 ){
			tmp = 1;
		}
*/
		for (loop2=0; loop2<nFirePointCnt; loop2++){
			if (loop1 == FirePointIDs[loop2]){
				tmp = 1;
printf("FirePoint (%ld, %ld)\n", pAllBuilding->x(), pAllBuilding->y() );		
			}
		}
		pAllBuilding->setIgnition(0, tmp);
		result = fread2( &tmp, sizeof( S32 ), 1, fpBuilding );
		pAllBuilding->setFieryness(0, tmp);
		result = fread2( &tmp, sizeof( S32 ), 1, fpBuilding );
		pAllBuilding->setBrokenness(0, tmp);

		result = fread2( &nCntRec2, sizeof( S32 ), 1, fpBuilding );
		for (loop2=0; loop2<nCntRec2; loop2++){
			result = fread2( &tmp, sizeof( S32 ), 1, fpBuilding );
			pNode[tmp-1].testConnect(0, pAllBuilding);
		}

		result = fread2( &tmp, sizeof( S32 ), 1, fpBuilding );
//		pAllBuilding->setBuildingShapeId(0, tmp);
		result = fread2( &tmp, sizeof( S32 ), 1, fpBuilding );
		pAllBuilding->setBuildingAreaGround(0, tmp);
		result = fread2( &tmp, sizeof( S32 ), 1, fpBuilding );
		pAllBuilding->setBuildingAreaTotal(0, tmp);
		result = fread2( &tmp, sizeof( S32 ), 1, fpBuilding );
		pAllBuilding->setBuildingCode(0, tmp);

		result = fread2( &nCntRec2, sizeof( S32 ), 1, fpBuilding );
		bytes.clear();
		for (loop2=0; loop2<nCntRec2; loop2++){

			result = fread2( &tmp, sizeof( S32 ), 1, fpBuilding );
			tmp_byte = tmp & 0xff000000;
			bytes.push_back( (Byte)(tmp_byte >> 24) );
			tmp_byte = tmp & 0x00ff0000;
			bytes.push_back( (Byte)(tmp_byte >> 16) );
			tmp_byte = tmp & 0x0000ff00;
			bytes.push_back( (Byte)(tmp_byte >> 8) );
			tmp_byte = tmp & 0x000000ff;
			bytes.push_back( (Byte)tmp_byte );

			result = fread2( &tmp, sizeof( S32 ), 1, fpBuilding );
			tmp_byte = tmp & 0xff000000;
			bytes.push_back( (Byte)(tmp_byte >> 24) );
			tmp_byte = tmp & 0x00ff0000;
			bytes.push_back( (Byte)(tmp_byte >> 16) );
			tmp_byte = tmp & 0x0000ff00;
			bytes.push_back( (Byte)(tmp_byte >> 8) );
			tmp_byte = tmp & 0x000000ff;
			bytes.push_back( (Byte)tmp_byte );
		}
		pAllBuilding->setBuildingApexes(0, bytes);
		
		pool.testAdd( pAllBuilding );
	}
	fclose(fpBuilding);

	//make AllGISDataList
	if( AllListOutput != 0 ){
		if( (fpTxt = fopen( "road.txt", "w+t" )) != NULL ){
			for ( loop1=0; loop1<nCntRoad; loop1++ ){
				sprintf(buffer, "%ld : %ld->%ld\n", pRoad[loop1].id() ,
					pRoad[loop1].head()->id(),	pRoad[loop1].tail()->id() );		
				result = fwrite( buffer, sizeof( char ), strlen( buffer ), fpTxt );
			}
			fclose(fpTxt);
		}

		if( (fpTxt = fopen( "node.txt", "w+t" )) != NULL ){
			for ( loop1=0; loop1<nCntNode; loop1++ ){
				sprintf(buffer, "%ld : (%ld, %ld)\n", pNode[loop1].id() ,
					pNode[loop1].x() ,pNode[loop1].y() );		
				result = fwrite( buffer, sizeof( char ), strlen( buffer ), fpTxt );
			}
			fclose(fpTxt);
		}

		if( (fpTxt = fopen( "building.txt", "w+t" )) != NULL ){
			for ( loop1=0; loop1<nCntBuilding; loop1++ ){
				sprintf(buffer, "%ld : (%ld, %ld)\n", pBuilding[loop1].id() ,
					pBuilding[loop1].x(), pBuilding[loop1].y() );		
				result = fwrite( buffer, sizeof( char ), strlen( buffer ), fpTxt );
			}
			fclose(fpTxt);
		}
	
	}

//	int k, z;

//	k = number;
//	z = nCntRoad / k;
//	if(z < 1)
//		z = 1;
//	for ( loop1=0; loop1<nCntRoad; loop1++ ){
//		if(loop1 % z == 0 && k-- > 0) {
//			Civilian* civilian = new Civilian();
//			pool.testAdd(civilian);
//			civilian->setPosition(0, &pRoad[loop1]);
//			civilian->setPositionExtra(0, pRoad[loop1].length() * 3 / 10);
//		}
//	}

	// Place Agents
	S32			AgentNum, AgentBNum, AgentRNum, AgentNNum;
	std::vector<S32> AgentBIDs;
	std::vector<S32> AgentRIDs;
	std::vector<S32> AgentNIDs;
	
	// Civilian
	if (bIniFileF == true)
		fpTxt = fopen(CivilianPoint_FN, "r+t");

	if (bIniFileF == true && fpTxt != NULL){
		while(fgets(buffer, size, fpTxt) != NULL) {
			if (buffer[0] == '#' || buffer[0] == '\n')	continue;
			int n = sscanf(buffer, "%s %s", name1, name2);
//			if ( strcmp( name1, "r") == 0 )
//			if ( strcmp( name1, "b") == 0 )
//			if ( strcmp( name1, "n") == 0 )
		}
		fclose(fpTxt);
	}
	else {
		S32			CivilianRatio = 20;

		if ( nCntRoad/CivilianRatio == 0 ){
			AgentNum = AgentBNum = 1;
			AgentRNum = 0;
		}
		else {
			AgentNum = 2* ( nCntBuilding / CivilianRatio );
			AgentBNum = AgentNum * 19 / 20;
			AgentRNum = AgentNum - AgentBNum;
		}

		// Civilian in Building
		DetPositions(&AgentBIDs, AgentBNum, nCntBuilding);
		printf("Civilian in Building:%ld\n", AgentBNum );
		for ( loop1=0; loop1<AgentBNum; loop1++ ){
			Civilian* civilian = new Civilian();
			pool.testAdd(civilian);
			civilian->setPosition(0, &pBuilding[AgentBIDs[loop1]]);
//			printf("CivilianPoint (%ld, %ld)\n", 
//				pBuilding[AgentBIDs[loop1]].x(), pBuilding[AgentBIDs[loop1]].y() );		
		}

		// Civilian on Road
		DetPositions(&AgentRIDs, AgentRNum, nCntRoad);
		printf("Civilian on Road:%ld\n", AgentRNum );
		for ( loop1=0; loop1<AgentRNum; loop1++ ){
			Civilian* civilian = new Civilian();
			pool.testAdd(civilian);
			civilian->setPosition(0, &pRoad[AgentRIDs[loop1]]);
			civilian->setPositionExtra(0, pRoad[AgentRIDs[loop1]].length() * 3 / 10);
//			printf("CivilianPoint s(%ld, %ld)->e(%ld, %ld)\n", 
//				pRoad[AgentRIDs[loop1]].head()->x() ,pRoad[AgentRIDs[loop1]].head()->y(),
//				pRoad[AgentRIDs[loop1]].tail()->x() ,pRoad[AgentRIDs[loop1]].tail()->y()	);		
		}

	}

	//AmbulanceTeam on Node
	if ( nCntNode/50 == 0 )
		AgentNNum = 1;
	else if ( nCntNode / 50 > 5 )
		AgentNNum = 5;
	else
		AgentNNum = nCntNode / 50;

	DetPositions(&AgentNIDs, AgentNNum, nCntNode);

	printf("AmbulanceTeam on Node:%ld\n", AgentNNum );
	for ( loop1=0; loop1<AgentNNum; loop1++ ){
		AmbulanceTeam* ambulance = new AmbulanceTeam();
		pool.testAdd(ambulance);
		ambulance->setPosition(0, &pNode[AgentNIDs[loop1]]);
//		printf("AmbulanceTeamPoint (%ld, %ld)\n", 
//			pNode[AgentNIDs[loop1]].x() ,pNode[AgentNIDs[loop1]].y() );		
	}

	//FireBrigade on Node
	if ( nCntNode / 50 == 0 )
		AgentNNum = 1;
	else if ( nCntNode / 50 > 10 )
		AgentNNum = 10;
	else
		AgentNNum = nCntNode / 50;

	DetPositions(&AgentNIDs, AgentNNum, nCntNode);

	printf("FireBrigade on Node:%ld\n", AgentNNum );
	for ( loop1=0; loop1<AgentNNum; loop1++ ){
		FireBrigade* fireteam = new FireBrigade();
		pool.testAdd(fireteam);
		fireteam->setPosition(0, &pNode[AgentNIDs[loop1]]);
//		printf("FireBrigadePoint (%ld, %ld)\n", 
//			pNode[AgentNIDs[loop1]].x() ,pNode[AgentNIDs[loop1]].y() );		
	}

	//PoliceForce on Road
	if ( nCntNode / 50 == 0 )
		AgentRNum = 1;
	else if ( nCntNode / 50 > 10 )
		AgentRNum = 10;
	else
		AgentRNum = nCntNode / 50;

	DetPositions(&AgentRIDs, AgentRNum, nCntRoad);

	printf("PoliceForce on Node:%ld\n", AgentRNum );
	for ( loop1=0; loop1<AgentRNum; loop1++ ){
		PoliceForce* policeforce = new PoliceForce();
		pool.testAdd(policeforce);
		policeforce->setPosition(0, &pRoad[AgentRIDs[loop1]]);
		policeforce->setPositionExtra(0, pRoad[AgentRIDs[loop1]].length() * 6 / 10);
//		printf("PoliceForcePoint s(%ld, %ld)->e(%ld, %ld)\n", 
//			pRoad[AgentRIDs[loop1]].head()->x() ,pRoad[AgentRIDs[loop1]].head()->y(),
//			pRoad[AgentRIDs[loop1]].tail()->x() ,pRoad[AgentRIDs[loop1]].tail()->y()	);		
	}

//#if 0
//	k = number;
//	z = nCntNode / k;
//	if(z < 1)
//		z = 1;
//	for ( loop1=0; loop1<nCntNode; loop1++ ){
//		if(loop1 % z == 0 && k-- > 0) {
//			AmbulanceTeam* ambulance = new AmbulanceTeam();
//			pool.testAdd(ambulance);
//			ambulance->setPosition(0, &pNode[loop1]);
//		}
//	}
//#endif
}

