
#include "common.hxx"
#include "basic.hxx"
#include "Config.hxx"
#include "Address.hxx"
#include "Input.hxx"
#include "Output.hxx"
#include "LongUDPSocket.hxx"
#include "ObjectPool.hxx"
#include "objects.hxx"
using namespace Rescue;

int main2(int argc, char** argv);
void commands(S32& time, Input& input, Output& output, LongUDPSocket& socket, Address& to, ObjectPool& pool);
void commands(S32& time, Header type, Input& input, ObjectPool& pool);
void move(S32& time, MovingObject* agent, Input& input, ObjectPool& pool);

int main(int argc, char** argv)
{
	try {
		return main2(argc, argv);
	} catch(std::exception& e) {
		fprintf(stderr, "%s\n", (const char*)e.what());
	} catch(...) {
		fprintf(stderr, "error...\n");
	}
	return 1;
}

int main2(int argc, char** argv)
{
	const char* host = "localhost";
	Config config;

	// 
	int i;
	int count = 0;
	for(i=1; i<argc; i++) {
		if(argv[i][0] == '-') {
			if(strcmp(argv[i], "-file") == 0) {
				if(++i < argc) {
					config.readConfigFile(argv[i]);
				} else {
					fprintf(stderr, "error: '-file'\n");
				}
			} else {
				if(++i < argc) {
					if(!config.setValue(argv[i-1]+1, argv[i]))
						fprintf(stderr, "error: '%s'\n", argv[i]);
				} else {
					fprintf(stderr, "error: '%s'\n", argv[i-1]);
				}
			}
		} else {
			switch(count++) {
			case 0:
				host = argv[i];
				break;
			default:
				fprintf(stderr, "error: '%s'\n", argv[i]);
				break;
			}
		}
	}

	// \Pbg쐬
	Address address = Address(INADDR_ANY, config.port(), true);
	LongUDPSocket socket(address, config.textlogname_gis());
	Input input;
	Output output;
	output.setWait(config.send_udp_wait());
	output.resetFrameSize(config.send_udp_size());
	Address to(host, config.port());	//M

	// Ot@CJ
	FILE* fp = fopen(config.logname().c_str(), "r"FOPEN_MODE_BINARY);
	if(fp == 0) {
		fprintf(stderr, "error: can't open file '%s'\n", config.logname().c_str());
		return 1;
	}
	fseek(fp, 0x20, SEEK_SET);
	output.clear();
	Byte b[4];
	fread(b, sizeof(b[0]), COUNT_OF(b), fp);	// TODO: G[`FbN
	S32 size = ((S32)b[0] << 24) | ((S32)b[1] << 16) | ((S32)b[2] << 8) | (S32)b[3];
	if(size % 4 != 0) {
		fprintf(stderr, "error: illegal data\n");
		return 1;
	}
	Byte* buffer = new Byte[size];
	fread(buffer, sizeof(Byte), size, fp);
	output.clear();
	for(i=0; i<size; i+=4) {
		Byte* b = buffer + i;
		S32 data = ((S32)b[0] << 24) | ((S32)b[1] << 16) | ((S32)b[2] << 8) | (S32)b[3];
		output.put(data);
	}
	delete[] buffer;
	// TODO: ͉


	printf("Waiting connection. Start viewer.\n");

	// ڑ҂
	bool innocence = true;
	while(innocence) {
		try {
			socket.receive(input);
			for(;;) {
				Header header = input.get();
				if(header == HEADER_NULL)
					break;

				S32 size = input.get();
				Input::Cursor start = input.cursor();
				switch(header) {
				default:
					printf("Unknown header.\n");
					break;
				case SK_CONNECT:
				case VK_CONNECT:
					printf("VK_CONNECT/SK_CONNECT\n");
					to = socket.addressRecievedFrom();
					socket.send(output, to);
					innocence = false;
					break;
				}

				input.setCursor(start);
				input.skip(size);
			}
		} catch(std::exception& e) {
			fprintf(stderr, "%s\n", (const char*)e.what());
		} catch(...) {
			fprintf(stderr, "error...\n");
		}
	}

	// ack҂
	bool connected = false;
	while(!connected) {
		try {
			socket.send(output, to);
			std::vector<const LongUDPSocket*> sockets;
			sockets.push_back(&socket);
			bool timeout = !LongUDPSocket::wait(sockets, 5 * config.step());
			if(timeout) {
				socket.send(output, to);
			} else {
				socket.receive(input);
				for(;;) {
					Header header = input.get();
					if(header == HEADER_NULL)
						break;

					S32 size = input.get();
					Input::Cursor start = input.cursor();
					switch(header) {
					default:
						printf("Unknown header.\n");
						break;
					case SK_ACKNOWLEDGE:
						printf("VK_ACKNOWLEDGE\n");
						connected = true;
						break;
					}

					input.setCursor(start);
					input.skip(size);
				}
			}
		} catch(std::exception& e) {
			fprintf(stderr, "%s\n", (const char*)e.what());
		} catch(...) {
			fprintf(stderr, "error...\n");
		}
	}

	printf("push enter key to start.\n");
	char dummy[10];
	fgets(dummy, COUNT_OF(dummy), stdin);

	// Iɐis󋵂𑗐M
	for(; !feof(fp); usleep(config.step() * 1000)) {
		output.clear();
		Byte b[4];
		fread(b, sizeof(b[0]), COUNT_OF(b), fp);	// TODO: G[`FbN
		S32 size = ((S32)b[0] << 24) | ((S32)b[1] << 16) | ((S32)b[2] << 8) | (S32)b[3];
		if(size % 4 != 0) {
			fprintf(stderr, "error: illegal data\n");
			return 1;
		}
		Byte* buffer = new Byte[size];
		fread(buffer, sizeof(Byte), size, fp);
		output.clear();
		for(int i=0; i<size; i+=4) {
			Byte* b = buffer + i;
			S32 data = ((S32)b[0] << 24) | ((S32)b[1] << 16) | ((S32)b[2] << 8) | (S32)b[3];
			output.put(data);
		}
		socket.send(output, to);
		delete[] buffer;
		// TODO: ͉
	}
	
	fclose(fp);

	return 0;
}
