
#include "common.hxx"
#include "basic.hxx"
#include "Config.hxx"
#include "Address.hxx"
#include "Input.hxx"
#include "Output.hxx"
#include "LongUDPSocket.hxx"
#include "ObjectPool.hxx"
#include "objects.hxx"
#include "misc_class.hxx"

using namespace Rescue;

int main2(int argc, char** argv);
void commands(S32& time, Input& input, Output& output, LongUDPSocket& socket, Address& to, ObjectPool& pool, Config& config);
void commands(S32& time, Header type, Input& input, ObjectPool& pool, Config& config);
void load(S32& time, MovingObject* agent, Input& input, ObjectPool& pool);
void unload(S32& time, MovingObject* agent, Input& input, ObjectPool& pool);
void rescue(S32& time, MovingObject* agent, Input& input, ObjectPool& pool);
void stretch(S32& time, MovingObject* agent, Input& input, ObjectPool& pool, Config& config);
void move(S32& time, MovingObject* agent, Input& input, ObjectPool& pool, Config& config);
void clear(S32& time, MovingObject* agent, Input& input, ObjectPool& pool, Config& config);
void step(S32& time, ObjectPool& pool, Config& config, Dice& dice, IMOCivilian& imoc);
void survive(S32& time, S32& x, Civilian* c);
void getHumanoidList(ObjectPool& pool, ObjList& human);
void getCivilianList(ObjectPool& pool, ObjList& civil);
void getCivilianList(ObjectPool& pool, ObjList& civil, ObjPosition& civilPos);
void getCivilianList(ObjList& human, ObjList& civil);
void getHumanoidList(ObjectPool& pool, Objects& humanoid);
void getCivilianList(ObjectPool& pool, Objects& civilian);
void getCivilianList(Objects& humanoid, Objects& civilian);

void testPoolRevise(ObjectPool& pool, IMOCivilian& imoc);
bool testPosition(IMOCivilian& imoc, MovingObject* mo);
void testSelectObject(ObjectPool& pool, IMOCivilian& imoc);
void testJudgeObject(IMOCivilian& imoc, MovingObject* mo);

int main(int argc, char** argv)
{
     try {
	  return main2(argc, argv);
     } catch(std::exception& e) {
	  fprintf(stderr, "%s\n", (const char*)e.what());
     } catch(...) {
	  fprintf(stderr, "error...\n");
     }
     return 1;
}

int main2(int argc, char** argv)
{
     const char* host = "localhost";
     Config config;

     // 
     int i;
     int count = 0;
     for(i=1; i<argc; i++) {
	  if(argv[i][0] == '-') {
	       if(strcmp(argv[i], "-file") == 0) {
		    if(++i < argc) {
			 config.readConfigFile(argv[i]);
		    } else {
			 fprintf(stderr, "error: '-file'\n");
		    }
	       } else {
		    if(++i < argc) {
			 if(!config.setValue(argv[i-1]+1, argv[i]))
			      fprintf(stderr, "error: '%s'\n", argv[i]);
		    } else {
			 fprintf(stderr, "error: '%s'\n", argv[i-1]);
		    }
	       }
	  } else {
	       switch(count++) {
	       case 0:
		    host = argv[i];
		    break;
	       default:
		    fprintf(stderr, "error: '%s'\n", argv[i]);
		    break;
	       }
	  }
     }

     // \Pbg쐬
     LongUDPSocket socket(config.textlogname_miscsimulator());
     Input input;
     Output output;
     output.setWait(config.send_udp_wait());
     output.resetFrameSize(config.send_udp_size());
     Address to(host, config.port());	//M

     // ڑbZ[W𑗐M
     printf("connecting...\n");
     output.clear();								//o̓obt@ɂ
     output.put(SK_CONNECT);						//o̓obt@ SK_CONNECT (4oCg)o͂
     Output::Cursor base = output.put(~(S32)0);	//body ̃TCYƂĂƂ肠_~[̒lo͂ĂāAbase ɂ̈ʒuLB
     output.put(0);								//o̓obt@ 0 o͂
     output.setSize(base);						//base ŋLʒuɁAbase 炱܂ł put oCgݒ肷B
     output.put(HEADER_NULL);
     socket.send(output, to, -1);					//output  put oCg to ֑MB

     // bZ[W[v
     S32 time = 0;
     ObjectPool pool;
     CivilPool civilpool;
     Dice dice;
     IMOCivilian imoc;

     for(;;) {
	  try {
	       socket.receive(input);	// TODO: G[OŕԂ	//f[^MAinput ֒~B
	       for(;;) {
		    Header header = input.get();	//input (4oCg)f[^oAheader ֑
		    if(header == HEADER_NULL)
			 break;

		    S32 size = input.get();
		    Input::Cursor start = input.cursor();	//ݓǂݏoĂʒuL

		    switch(header) {
		    default:
			 printf("Unknown header.\n");
			 break;
		    case KS_CONNECT_ERROR:
			 printf("Connection failed.\n");
			 return 1;
		    case KS_CONNECT_OK:
			 printf("Connect Ok.\n");
			 // ack Ԃ
			 output.clear();
			 output.put(SK_ACKNOWLEDGE);
			 output.put(~(S32)0);
			 output.put(HEADER_NULL);
			 socket.send(output, to, -1);
					
			 to = socket.addressRecievedFrom();	//ȍ~̑MύX
			 pool.restructure(0, input);			//[hfB
			 testSelectObject(pool, imoc);

			 break;
		    case KS_COMMANDS:
			 printf("KS_COMMANDS\n");
			 commands(time, input, output, socket, to, pool, config);
			 break;
		    case KS_UPDATE:
			 time = input.get();
			 pool.restructure(time, input);
			 //civilpool.restructure(time, pool);
			 printf("time = %ld.\n", (long)time);
			 //testPoolRevise(pool, imoc);
			 step(++time, pool, config, dice, imoc);

			 break;
		    }

		    input.setCursor(start);
		    input.skip(size);						//size oCgXLbv
	       }
	  } catch(std::exception& e) {
	       fprintf(stderr, "%s\n", (const char*)e.what());
	  } catch(...) {
	       fprintf(stderr, "error...\n");
	  }
     }
	
     return 0;
}

void testPoolRevise(ObjectPool& pool, IMOCivilian& imoc)
{
     printf("---------Pool-------------\n");
     Objects::const_iterator it = pool.objects().begin();
     for(; it != pool.objects().end() ; it++) {
	  Civilian* c = dynamic_cast<Civilian*>(*it);
	  if(c != 0) {
	       printf("Id: %3d Pos: %3d\n", (int)c->id(), (int)c->position());
	  }
     }
     printf("---------imoc--------------\n");
     Objects::const_iterator it2 = imoc.getIndoor().begin();
     for(; it2 != imoc.getIndoor().end() ; it2++) {
	  Civilian* c = dynamic_cast<Civilian*>(*it2);
	  printf("Id: %3d Pos: %3d\n", (int)c->id(), (int)c->position());
     }
}

void testSelectObject(ObjectPool& pool, IMOCivilian& imoc)
{
     Objects::const_iterator it = pool.objects().begin();
     for(; it != pool.objects().end(); it++) {
	  MovingObject* mo = dynamic_cast<MovingObject*>(*it);
	  if(mo != 0)
	       testJudgeObject(imoc, mo);
     }
     
     // InWoodenInConcreteqIndoor̃Xg
     Objects::const_iterator p1 = imoc.getInWooden().begin();
     Objects::const_iterator p2 = imoc.getInWooden().end();
     Objects::const_iterator p3 = imoc.getInConcrete().begin();
     Objects::const_iterator p4 = imoc.getInConcrete().end();
     Objects& indoor = imoc.getIndoor();
     indoor.insert(indoor.end(), p1, p2);
     indoor.insert(indoor.end(), p3, p4);

}

void testJudgeObject(IMOCivilian& imoc, MovingObject* mo)
{
     switch(mo->type()) {
     case TYPE_BUILDING:
	  // printf("Id: %3d Building TypeID: %3d\n", (int)o->id(), (int)o->type());
	  break;
     case TYPE_REFUGE:
	  //printf("Id: %3d Refuge TypeID: %3d\n", (int)o->id(), (int)o->type());
	  break;
     case TYPE_AMBULANCE_CENTER:
	  //printf("Id: %3d AmbulanceCenter TypeID: %3d\n", (int)o->id(), (int)o->type());
	  break;
     case TYPE_POLICE_OFFICE:
	  //printf("Id: %3d PoliceOffice TypeID: %3d\n", (int)o->id(), (int)o->type());
	  break;
     case TYPE_FIRE_STATION:
	  //printf("Id: %3d FireStation TypeID: %3d\n", (int)o->id(), (int)o->type());
	  break;
     case TYPE_CIVILIAN:
	  testPosition(imoc, mo);
	  //printf("Id: %3d Civilian TypeID: %3d\n", (int)mo->id(), (int)mo->type());
	  break;
     case TYPE_FIRE_BRIGADE:
	  //printf("Id: %3d FireBrigade TypeID: %3d\n", (int)o->id(), (int)o->type());
	  break;
     case TYPE_AMBULANCE_TEAM:
	  //printf("Id: %3d AmbulanceTeam TypeID: %3d\n", (int)o->id(), (int)o->type());
	  break;
     case TYPE_POLICE_FORCE:
	  //printf("Id: %3d PoliceForce TypeID: %3d\n", (int)o->id(), (int)o->type());
	  break;
     case TYPE_CAR:
	  //printf("Id: %3d Car TypeID: %3d\n", (int)o->id(), (int)o->type());
	  break;
     case TYPE_WORLD:
	  break;
     case TYPE_RIVER:
	  break;
     case TYPE_RIVER_NODE:
	  break;
     case TYPE_NODE:
	  break;
     case TYPE_ROAD:
	  break;
     default:
	  break;
     }
}


bool testPosition(IMOCivilian& imoc, MovingObject* mo)
{
     // ɂ
     Building* b = dynamic_cast<Building*>(mo->position());
     if(b != 0) {
	  // ؑł
	  if(b->buildingAttributes() == 0)
	       imoc.getInWooden().push_back(mo);
	  // ؑł
	  else
	       imoc.getInConcrete().push_back(mo);
	  return true;
     }
     // Oɂ
     else {
	  imoc.getOutdoor().push_back(mo);
	  return false;
     }
}

#if 0
void makeDestinyArray(ObjectPool& pool)
{
     Objects::const_iterator it = pool.objects().begin();
     for(; it != pool.objects().end(); it++) {
     }
}

void getHumanoidList(ObjectPool& pool, ObjList& human)
{
     printf("== In void getHumanoidList(ObjectPool& pool, ObjList& human)\n");
     Objects::const_iterator it = pool.objects().begin();
     for(; it != pool.objects().end(); it++) {
	  Humanoid* h = dynamic_cast<Humanoid*>(*it);
	  if(h != 0) {
	       human.List().push_back();
	       human.incrNum();
	  }
	  Civilian* c = dynamic_cast<Civilian*>(*it);
	  if(c != 0) {
	  }
     }
}

void getCivilianList(ObjectPool& pool, ObjList& civil)
{
     printf("== In void getCivilianList(ObjectPool& pool, ObjList& civil)\n");
     Objects::const_iterator it = pool.objects().begin();
     for(; it != pool.objects().end(); it++) {
	  Civilian* c = dynamic_cast<Civilian*>(*it);
	  if(c != 0) {
	       civil.List().push_back();
	       civil.incrNum();
	  }
     }
}

void getCivilianList(ObjectPool& pool, ObjList& civil, ObjPosition& civilPos)
{
     printf("== In void getCivilianList(ObjectPool& pool, ObjList& civil, ObjPosition& civilPos)\n");
     S32 indoor = 0;
     S32 outdoor = 0;
     Objects::const_iterator it = pool.objects().begin();
     for(; it != pool.objects().end(); it++) {
	  Civilian* c = dynamic_cast<Civilian*>(*it);
	  if(c != 0) {
	       civil.List().push_back(c);
	       civil.incrNum();
	       if(dynamic_cast<Building*>(c->position()) != 0)
		    indoor++;
	       else
		    outdoor++;
	  }
     }

     civilPos.getIndoor().setTotal(indoor);
     civilPos.getIndoor().setDead(RATIO_DEAD * indoor);
     civilPos.getIndoor().setSerious(RATIO_SERIOUS * indoor);
     civilPos.getIndoor().setSlight(RATIO_SLIGHT * indoor);
     printf("indoor total = %d\n", (S32)civilPos.getIndoor().getTotal());
     printf("indoor dead = %d\n", (S32)civilPos.getIndoor().getDead());
     printf("indoor serious = %d\n", (S32)civilPos.getIndoor().getSerious());
     printf("indoor slight = %d\n", (S32)civilPos.getIndoor().getSlight());

     civilPos.getOutdoor().setTotal(outdoor);
     printf("outdoor total = %d\n", (S32)civilPos.getOutdoor().getTotal());
     printf("Civilian = %d\n", static_cast<int>(civil.getNum()));
}

void getCivilianList(ObjList& human, ObjList& civil)
{
     printf("== In void getCivilianList(ObjList& human, ObjList& civil)\n");
     Objects::const_iterator it = human.List().begin();
     for(; it != human.List().end(); it++) {
	  Civilian* c = dynamic_cast<Civilian*>(*it);
	  if(c != 0) {
	       civil.List().push_back(c);
	       civil.incrNum();
	  }
     }
}

void getHumanoidList(ObjectPool& pool, Objects& humanoid)
{
     printf("== In void getHumanoidList(ObjectPool& pool, Objects& humanoid)\n");
     Objects::const_iterator it = pool.objects().begin();
     for(; it != pool.objects().end(); it++) {
	  Humanoid* h = dynamic_cast<Humanoid*>(*it);
	  if(h != 0) {
	       humanoid.push_back(h);		  
	  }
     }
}

void getCivilianList(ObjectPool& pool, Objects& civilian)
{
     printf("== In void getCivilianList(ObjectPool& pool, Objects& civilian)\n");
     Objects::const_iterator it = pool.objects().begin();
     for(; it != pool.objects().end(); it++) {
	  Civilian* c = dynamic_cast<Civilian*>(*it);
	  if(c != 0) {
	       civilian.push_back(c);
	  }
     }
}

// Civilian can't be count
void getCivilianList(Objects& humanoid, Objects& civilian)
{
     printf("== In void getCivilianList(Objects& humanoid, Objects& civilian)\n");
     Objects::const_iterator it = humanoid.begin();
     for(; it != humanoid.end(); it++) {
	  Civilian* c = dynamic_cast<Civilian*>(*it);
	  if(c != 0) {
	       civilian.push_back(c);
	  }
     }
}
#endif

void commands(S32& time, Input& input, Output& output, LongUDPSocket& socket, Address& to, ObjectPool& pool, Config& config)
{
     // KS_COMMANDS M
     time = input.get();
     Header type;
     while(type = input.get(), type != HEADER_NULL) {
	  S32 size = input.get();
	  Input::Cursor start = input.cursor();
	  commands(time, type, input, pool, config);
	  input.setCursor(start);
	  input.skip(size);
     }

     // V~[Vʂ𑗐M
     output.clear();
     output.put(SK_UPDATE);
     Output::Cursor base = output.put(~(S32)0);
     pool.output(output, time);
     output.setSize(base);
     output.put(HEADER_NULL);
     socket.send(output, to, time);
}

void commands(S32& time, Header type, Input& input, ObjectPool& pool, Config& config)
{
     Id sender;
     while(sender = input.get(), sender != 0) {
	  S32 size = input.get();
	  Input::Cursor start = input.cursor();

	  Object* o = pool.get(sender);	//[hfAsender  ID ɎIuWFNgoB
	  MovingObject* agent = dynamic_cast<MovingObject*>(o);
	  if(agent != 0) {
	       switch(type) {
	       case AK_LOAD:
		    load(time, agent, input, pool);
		    break;
	       case AK_UNLOAD:
		    unload(time, agent, input, pool);
		    break;
	       case AK_RESCUE:
		    rescue(time, agent, input, pool);
		    break;
	       case AK_STRETCH:
		    stretch(time, agent, input, pool, config);
		    break;
	       case AK_MOVE:
		    move(time, agent, input, pool, config);
		    break;
	       case AK_CLEAR:
		    clear(time, agent, input, pool, config);
		    break;
	       }
	  }
		
	  input.setCursor(start);
	  input.skip(size);
     }
}

void load(S32& time, MovingObject* agent, Input& input, ObjectPool& pool)
{
     if(dynamic_cast<AmbulanceTeam*>(agent) != 0) {
	  if(agent->contents().empty()) {
	       Id id = input.get();
	       MovingObject* target = dynamic_cast<MovingObject*>(pool.get(id));
	       if(target != 0) {
		    Humanoid* h = dynamic_cast<Humanoid*>(target);
		    if((h != 0) && (h->buriedness() != 0))
			 printf("load failed %ld by %ld\n", (long)target->id(), (long)agent->id());
		    else if(target->position() == agent->position()) {
			 target->setPosition(time, agent);
			 target->setPositionExtra(time, 0);
			 printf("load %ld by %ld\n", (long)target->id(), (long)agent->id());
		    }
	       }
	  }
     }
}
void unload(S32& time, MovingObject* agent, Input& input, ObjectPool& pool)
{
     if(dynamic_cast<AmbulanceTeam*>(agent) != 0) {
	  if(!agent->contents().empty()) {
	       Object* target = agent->contents().front();
	       ASSERT(dynamic_cast<MovingObject*>(target) != 0);
	       MovingObject* mo = (MovingObject*)target;
	       mo->setPosition(time, agent->position());
	       mo->setPositionExtra(time, agent->positionExtra());
	       printf("unload %ld by %ld\n", (long)mo->id(), (long)agent->id());
	  }
     }
}
void rescue(S32& time, MovingObject* agent, Input& input, ObjectPool& pool)
{
     if(dynamic_cast<AmbulanceTeam*>(agent) != 0) {
	  Id id = input.get();
	  Humanoid* target = dynamic_cast<Humanoid*>(pool.get(id));
	  if(target != 0) {
	       if(target->position() == agent->position()) {
		    if(target->buriedness() > 0) {
			 target->setBuriedness(time, target->buriedness() - 1);
			 printf("rescue %ld by %ld\n", (long)target->id(), (long)agent->id());
		    }
		    else
			 target->setBuriedness(time, 0);
	       }
	  }
     }
}
void stretch(S32& time, MovingObject* agent, Input& input, ObjectPool& pool, Config& config)
{
     FireBrigade* brigade = dynamic_cast<FireBrigade*>(agent);
     if(brigade != 0) {
	  brigade->setStretchedLength(time, brigade->stretchedLength() + config.stretch_length());
	  printf("stretch by %ld\n", (long)agent->id());
     }
}
void move(S32& time, MovingObject* agent, Input& input, ObjectPool& pool, Config& config)
{
     FireBrigade* brigade = dynamic_cast<FireBrigade*>(agent);
     if(brigade != 0 && brigade->stretchedLength() > 1)
	  brigade->setStretchedLength(time, 0);
}
void clear(S32& time, MovingObject* agent, Input& input, ObjectPool& pool, Config& config)
{
     if(dynamic_cast<PoliceForce*>(agent) != 0) {
	  Id id = input.get();
	  Road* target = dynamic_cast<Road*>(pool.get(id));
	  if(target != 0) {
	       if(target != agent->position()	&& target->head() != agent->position() && target->tail() != agent->position())
		    return;
	       printf("clear %ld by %ld\n", (long)target->id(), (long)agent->id());
	       S32 block = target->block();
	       S32 repairCost = target->repairCost();
	       if(repairCost <= 0) {
		    block = 0;
	       } else {
		    block = block * (repairCost - 1) / repairCost;
		    repairCost--;
		    target->setRepairCost(time, repairCost);
	       }
	       target->setBlock(time, block);
	       printf("clear %ld by %ld\n", (long)target->id(), (long)agent->id());
	  }
     }
}

void step(S32& time, ObjectPool& pool, Config& config, Dice& dice, IMOCivilian& imoc)
{
     Objects::const_iterator it = pool.objects().begin();
     for(; it != pool.objects().end(); it++) {
	  // PK̐is
	  Humanoid* h = dynamic_cast<Humanoid*>(*it);
	  if(h != 0) {
	       // ܂Ă
	       if(h->buriedness() != 0) {
		    printf("time %ld: buried ID %ld\n", (long)time, (long)h->id());
		    if( dice.castDice(0, 4096) <= 0.2 * 4096 ) {
			 S32 newHp = static_cast<S32>(0.9 * h->hp());
			 h->setHp(time, newHp);
		    } else if( dice.castDice(0, 4096) <= 0.4 * 4096 ) {
			 S32 newHp = static_cast<S32>(0.8 * h->hp());
			 h->setHp(time, newHp);
		    }
	       }
	       // _[W
	       if( h->damage() > 0 && time % 10 == 0) {
		    S32 p1 = dice.castDice(0, 4096);
		    if( p1 <= 0.1 * 4096 ) {
			 S32 newHp = static_cast<S32>(0.95 * h->hp());
			 h->setHp(time, newHp);
		    }
		    else if( p1 > 0.1 * 4096 && p1 <= 0.2 * 4096 ) {
			 S32 newHp = static_cast<S32>(0.90 * h->hp());
			 h->setHp(time, newHp);					
		    }
		    else if( p1 > 0.2 * 4096 && p1 <= 0.25 * 4096 ) {
			 S32 newHp = static_cast<S32>(0.85 * h->hp());
			 h->setHp(time, newHp);					
		    }
		    else if( p1 > 0.25 * 4096 && p1 <= 0.3 * 4096 ) {
			 S32 newHp = static_cast<S32>(0.80 * h->hp());
			 h->setHp(time, newHp);					
		    }
		    else if( p1 > 0.4 * 4096 && p1 <= 0.5 * 4096 ) {
			 S32 newHp = static_cast<S32>(0.75 * h->hp());
			 h->setHp(time, newHp);					
		    }
	       }
	       // ֓AÂsꂽB
	       if(dynamic_cast<Refuge*>(h->position()) != 0)
		    h->setDamage(time, 0);	
	       S32 newHp = h->hp() - h->damage();
	       if(newHp < 0) {
		    newHp = 0;
	       }
	       if(newHp != h->hp())
		    h->setHp(time, newHp);
	  }


	  Building* b = dynamic_cast<Building*>(*it);
	  if(b != 0) {
	       // |ɂ镉
	       if(b->brokenness() > b->oldBrokenness() &&
		  time == b->getTimeBrokennessUpdated()) {
		    // |󂵂A
		    Objects::const_iterator it2 = b->contents().begin();
		    for(; it2 != b->contents().end(); it2++) {
			 Humanoid* h = dynamic_cast<Humanoid*>(*it2);
			 if(h != 0) {
			      S32 p1 = dice.castDice(0, 4096);
			      if( p1 <= RATIO_SLIGHT * 4096 ) {
				   S32 newHp = static_cast<S32>(0.9 * h->hp());
				   h->setHp(time, newHp);
				   h->setDamage(time, DECAY_HURT_SLIGHT);
			      }
			      else if( p1 > RATIO_SLIGHT * 4096 && 
				       p1 <= (RATIO_SLIGHT + RATIO_SERIOUS) * 4096) {
				   S32 newHp = static_cast<S32>(0.4 * h->hp());
				   h->setHp(time, newHp);
				   h->setDamage(time, DECAY_HURT_SERIOUS);
			      }
			      else {
				   S32 newHp = static_cast<S32>(1.0 * h->hp());
				   h->setHp(time, newHp);
			      }

			      AmbulanceTeam* a = dynamic_cast<AmbulanceTeam*>(h);
			      if(a != 0) {
				   S32 p2 = dice.castDice(0, 4096);
				   if( p2 <= 0.05 * 4096 )
					h->setBuriedness(time, 10);
			      }
			      else {
				   S32 p3 = dice.castDice(0, 4096);
				   if( p3 <= 0.3 * 4096 )
					h->setBuriedness(time, 10);
			      }
			      h->setDamage(time, DECAY_BURIED);	// G[WFgփ_[W^B
			 }
		    }
	       }
	       // ΍Ђɂ镉
	       if(1 <= b->fieryness() && b->fieryness() <= 3) {
		    // RĂA
		    Objects::const_iterator it2 = b->contents().begin();
		    for(; it2 != b->contents().end(); it2++) {
			 Humanoid* h = dynamic_cast<Humanoid*>(*it2);
			 if(h != 0) {
			      h->setDamage(time, 1000);	// G[WFgփ_[W^B
			      S32 newHp = static_cast<S32>(0.8 * h->hp());
			      h->setHp(time, newHp);
			 }
		    }
	       }
	  }
     }

     // nk̏_[W
     if(time == 2) {
	  S32 deadNum = 0;
	  S32 seriousNum = 0;
	  S32 slightNum = 0;
	  S32 indoorNum = imoc.getIndoor().size();
//	  printf("indoorNum: %3d\n", (int)indoorNum);
	  do {
	       Objects::const_iterator it3 = imoc.getIndoor().begin();
	       for(; it3 != imoc.getIndoor().end() && (deadNum < indoorNum * RATIO_DEAD || seriousNum < indoorNum * RATIO_SERIOUS || slightNum < indoorNum * RATIO_SLIGHT); it3++) {
		    Civilian* c = dynamic_cast<Civilian*>(*it3);
		    if(c != 0 && c->hp() == 10000) {
			 Building* b = dynamic_cast<Building*>(c->position());
			 if(b->brokenness() > b->oldBrokenness()) {
			      S32 p4 = dice.castDice(0, 4096);
			      if( p4 <= 0.1 * 4096 ) {
				   if(deadNum < indoorNum * RATIO_DEAD) {
					c->setHp(time, 0);
					c->setDamage(time, 0);
					deadNum++;
//					printf("dead Id: %3d\n", (int)c->id());
				   }
			      }
			      else if(p4 > 0.1 * 4096 && p4 <= 0.2 * 4096) {
				   if(seriousNum < indoorNum * RATIO_SERIOUS) {
					S32 newHp = static_cast<S32>(0.4 * c->hp());
					c->setHp(time, newHp);
					c->setDamage(time, DECAY_HURT_SERIOUS);
					seriousNum++;
//					printf("DECAY_HURT_SERIOUS: %3d\n", (int)DECAY_HURT_SERIOUS);
//					printf("serious Id: %3d damage: %3d\n", (int)c->id(), (int)c->damage());
				   }
			      }
			      else if(p4 > 0.2 * 4096 && p4 <= 0.4 * 4096) {
				   if(slightNum < indoorNum * RATIO_SLIGHT) {
					S32 newHp = static_cast<S32>(0.7 * c->hp());
					c->setHp(time, newHp);
					c->setDamage(time, DECAY_HURT_SLIGHT);
					slightNum++;
//					printf("DECAY_HURT_SLIGHT: %3d\n", (int)DECAY_HURT_SLIGHT);
//					printf("slight Id: %3d damage: %3d\n", (int)c->id(), (int)c->damage());
				   }
			      }
			 }
		    }
		    printf("dead: %3d serious: %3d slight: %3d\n", (int)deadNum, (int)seriousNum, (int)slightNum);
	       }
	  } while(deadNum < indoorNum * RATIO_DEAD || seriousNum < indoorNum * RATIO_SERIOUS || slightNum < indoorNum * RATIO_SLIGHT);

	  Objects::const_iterator it4 = imoc.getIndoor().begin();
	  for(; it4 != imoc.getIndoor().end(); it4++) {
	       Civilian* c = dynamic_cast<Civilian*>(*it4);
	       if(c != 0 && c->buriedness() == 0) {
		    AmbulanceTeam* a = dynamic_cast<AmbulanceTeam*>(c);
		    if(a != 0) {
			 S32 p2 = dice.castDice(0, 4096);
			 if( p2 <= 0.05 * 4096 ) {
			      c->setBuriedness(time, 10);
			      if(c->damage() < DECAY_BURIED)
				   c->setDamage(time, DECAY_BURIED);	// G[WFgփ_[W^B
			 }
		    }
		    else {
			 S32 p3 = dice.castDice(0, 4096);
			 if( p3 <= 0.3 * 4096 ) {
			      c->setBuriedness(time, 10);
			      if(c->damage() < DECAY_BURIED)
				   c->setDamage(time, DECAY_BURIED);	// G[WFgփ_[W^B
			 }
		    }
	       }
	  }
     }
}
