// misc_class.hxx
#ifndef ___MISC_CLASS
#define ___MISC_CLASS

namespace Rescue{
	
     typedef S32 HumanoidState;
     const HumanoidState STATE_UNHURT  = 0;
     const HumanoidState STATE_SLIGHT  = 1;
     const HumanoidState STATE_SERIOUS = 2;
     const HumanoidState STATE_DEAD    = 3;
	
     typedef double HurtRatio;
     const HurtRatio RATIO_SERIOUS = .00967;
     const HurtRatio RATIO_SLIGHT  = .0258;
     const HurtRatio RATIO_DEAD    = .00924; // (= 0.924 %)
	
     const double BURIED_RATIO     = .1;
	
     typedef S32 BuriedState;
     const BuriedState NON_BURIED = 0;
     const BuriedState BURIED     = 1;
	
     const S32 DECAY_HURT_SERIOUS  = 40;
     const S32 DECAY_HURT_SLIGHT  = 10;
     const S32 DECAY_BURIED  = 40;
     const S32 DECAY_FIERY  = 400;
	
     typedef std::vector<HumanoidState> HurtDegree;
     typedef std::vector<BuriedState>   Buried;

     class IMOCivilian {
     private:
	  Objects inWooden;
	  Objects inConcrete;
	  Objects indoor;
	  Objects outdoor;
     public:
	  IMOCivilian(): inWooden(), inConcrete(), indoor(), outdoor() {
	  }
	  Objects& getInWooden() { return inWooden; }
	  Objects& getInConcrete() { return inConcrete; }
	  Objects& getIndoor() { return indoor; }	  
	  Objects& getOutdoor() { return outdoor; }
     };

     class Probability {
     private:
	  double prob;
     public:
	  Probability() {
	       prob = 1.0;
	  }
	  double getProb() {
	       return prob;
	  }
	  void setProb(double p) {
	       prob = p;
	  }
	  double getSurvivalProb(S32 time) {
	       double tmp = -0.0293 * (time / 60) + 0.00177;
	       double result = exp(tmp);
	       return (result);
	  }
     };
	
     class Dice {
     private:
	  S32 seed;
     public:
	  Dice() {
	       seed = time(NULL);
	       srand(seed);
	  }
	  Dice(S32 s) {
	       seed = s;
	       srand(seed);
	  }
	  void setSeed(S32 s) {
	       seed = s;
	  }
	  void initiRandom() {
	       srand(seed);
	  }
		
	  S32 castDice(S32 begin, S32 end) {
	       S32 range = end - begin + 1;
	       return ( begin + rand() / ( RAND_MAX / range + 1) );
	  }
     };
	
     class ObjList {
		
     private:
	  Objects list;
	  S32 number;    // a number of the member of 'Object list'
		
     public:
	  ObjList() : list() {
	       number = 0;
	  }
	  Objects& List() { return (list); }
	  S32 getNum() { return (number); }
	  void incrNum() { ++number; }
	  void decrNum() { --number; }
     };
	
     class ObjNumInfo {
     private:
	  S32 total;
	  S32 dead;
	  S32 serious;
	  S32 slight;
	  S32 unhurt;
	  S32 buried;
     public:
	  ObjNumInfo() {
	       total = 0;
	       dead = 0;
	       serious = 0;
	       slight = 0;
	       unhurt = 0;
	       buried = 0;
	  }
	  S32 getTotal() { return (total); }
	  S32 getDead() { return (dead); }
	  S32 getSerious() { return (serious); }
	  S32 getSlight() { return (slight); }
	  S32 getUnhurt() { return (unhurt); }
	  S32 getBuried() { return (buried); }
	  void increTotal() { total++; }
	  void setTotal(S32 number) { total = number; }
	  void setDead(S32 number) { dead = number; }
	  void setSerious(S32 number) { serious = number; }
	  void setSlight(S32 number) { slight = number; }
	  void setUnhurt(S32 number) { unhurt = number; }
	  void setBuried(S32 number) { buried = number; }
	  void setAll(S32 t, S32 d, S32 se, S32 sl, S32 b) {
	       total = t;
	       dead = d;
	       serious = se;
	       slight = sl;
	       unhurt = t - d - se - sl;
	       buried = b;
	  }
     };
	
     class ObjInitPos {
     private:
	  Objects civil;
	  ObjNumInfo dist;
	  HurtDegree hurtdeg;
	  Buried buried;
     public:
	  ObjInitPos() : civil(), dist(), hurtdeg() {
	  }
	  Objects& getCivil() { return civil; }
	  ObjNumInfo& getDist() { return dist; }
	  HurtDegree& getHurtdeg() { return hurtdeg; }
	  Buried& getBuried() { return buried; }
		
	  void setBuried() {
	       S32 cnt = dist.getBuried();
	       for(S32 i = 0; i < dist.getTotal(); i++) {
		    if(cnt != 0) {
			 buried.push_back(BURIED);
			 cnt--;
		    }
		    else
			 buried.push_back(NON_BURIED);
	       }
	  }
		
	  void shuffleBuried() {
	       Dice dice;
/*	       printf("Buried\n");*/
		   S32 i;
	       for(i = 0; i < static_cast<S32>(buried.size()); i++) {
		    S32 safix = dice.castDice(0, dist.getTotal() - 1);
		    HumanoidState tmp = buried[i];
		    buried[i] = buried[safix];
		    buried[safix] = tmp;
	       }
	       for(i = buried.size() - 1; i >= 0; i--) {
		    S32 safix = dice.castDice(0, dist.getTotal() - 1);
		    HumanoidState tmp = buried[i];
		    buried[i] = buried[safix];
		    buried[safix] = tmp;
	       }
	       for(i = 0; i < static_cast<S32>(buried.size()); i++) {
//				printf("%d ", static_cast<int>(buried[i])) ;
	       }
//			printf("\n");
	  }
		
	  void setHurtDeg() {
	       S32 i;
	       for(i = 0; i < dist.getUnhurt(); i++) {
		    hurtdeg.push_back(STATE_UNHURT);
	       }
	       for(i = 0; i < dist.getSlight(); i++) {
		    hurtdeg.push_back(STATE_SLIGHT);
	       }
	       for(i = 0; i < dist.getSerious(); i++) {
		    hurtdeg.push_back(STATE_SERIOUS);
	       }
	       for(i = 0; i < dist.getDead(); i++) {
		    hurtdeg.push_back(STATE_DEAD);
	       }
	  }
		
	  void shuffleHurtDeg() {
	       Dice dice;
/*	       printf("HurtDeg\n");*/
		   S32 i;
	       for(i = 0; i < static_cast<S32>(hurtdeg.size()); i++) {
		    S32 safix = dice.castDice(0, dist.getTotal() - 1);
		    HumanoidState tmp = hurtdeg[i];
		    hurtdeg[i] = hurtdeg[safix];
		    hurtdeg[safix] = tmp;
	       }
	       for(i = hurtdeg.size() - 1; i >= 0; i--) {
		    S32 safix = dice.castDice(0, dist.getTotal() - 1);
		    HumanoidState tmp = hurtdeg[i];
		    hurtdeg[i] = hurtdeg[safix];
		    hurtdeg[safix] = tmp;
	       }
	       for(i = 0; i < static_cast<S32>(hurtdeg.size()); i++) {
//				printf("%d ", static_cast<int>(hurtdeg[i])) ;
	       }
//			printf("\n");
			
	  }
     };
	
     class CivilPool {
     private:
	  ObjInitPos indoor;
	  ObjInitPos inWooden;
	  ObjInitPos inConcrete;
	  ObjInitPos outdoor;
	  // indoor{wooden, concrete}
     public:
	  CivilPool() : indoor(), inWooden(), inConcrete(), outdoor() {
	  }
	  ObjInitPos& getIndoor() { return indoor; }
	  ObjInitPos& getInWooden() { return inWooden; }
	  ObjInitPos& getInConcrete() { return inConcrete; }
	  ObjInitPos& getOutdoor() { return outdoor; }
		
	  void setDegreeAll() {
	       //      printf("==indoor\n");
	       setDegree(indoor);
	       //      printf("==inWooden\n");
	       setDegree(inWooden);
	       //      printf("==inConcrete\n");
	       setDegree(inConcrete);
	       //      printf("==outdoor\n");
	       setDegree(outdoor);
	  }
		
	  void setDegree(ObjInitPos& pos) {
	       pos.setHurtDeg();
	       pos.setBuried();
	       pos.shuffleHurtDeg();
	       pos.shuffleBuried();
	  }
		
	  void setNumInfo(ObjInitPos& pos) {
	       S32 total, dead, serious ,slight, buried = 0;
	       total = pos.getDist().getTotal();
	       dead = (S32)(total * RATIO_DEAD);
	       serious = (S32)(total * RATIO_SERIOUS);
	       slight = (S32)(total * RATIO_SLIGHT);
	       buried = (S32)(total * BURIED_RATIO);
	       pos.getDist().setAll(total, dead, serious, slight, buried);
	  }
		
	  void setNumInofAll() {
	       setNumInfo(indoor);
	       setNumInfo(inWooden);
	       setNumInfo(inConcrete);
	       setNumInfo(outdoor);
	  }
		
	  void selectCivil(ObjInitPos& pos, Civilian* c) {
	       pos.getCivil().push_back(c);
	       pos.getDist().increTotal();
	  }
		
	  void selectCiovilAll(Civilian* c) {
	       selectCivil(indoor, c);
	       selectCivil(inWooden, c);
	       selectCivil(inConcrete, c);
	       selectCivil(outdoor, c);
	  }
		
		
	  void restructure(S32 time, ObjectPool& pool) {
	       if(time == 1)
		    structure(time, pool);
	  }
		
	  void structure(S32 time, ObjectPool& pool) {
	       Objects::const_iterator it = pool.objects().begin();
	       for(; it != pool.objects().end(); it++) {
		    Civilian* c = dynamic_cast<Civilian*>(*it);
		    if(c != 0) {
			 Building* b = dynamic_cast<Building*>(c->position());
			 if(b != 0) {
			      selectCivil(indoor, c);
			      if(b->buildingAttributes() == 0)
				   selectCivil(inWooden, c);
			      else
				   selectCivil(inConcrete, c);
			 }
			 else {
			      selectCivil(outdoor, c);
			 }
		    }
	       }
			
	       setNumInfo(indoor);      
	       setNumInfo(inWooden);
	       setNumInfo(inConcrete);
	       setNumInfo(outdoor);
#if 0
	       printf("inWooden\n");
	       printf("total   = %d\n", inWooden.getDist().getTotal());
	       printf("dead    = %d\n", inWooden.getDist().getDead());
	       printf("serious = %d\n", inWooden.getDist().getSerious());
	       printf("slight  = %d\n", inWooden.getDist().getSlight());
	       printf("unhurt  = %d\n", inWooden.getDist().getUnhurt());
	       printf("inConcrete\n");
	       printf("total   = %d\n", inConcrete.getDist().getTotal());
	       printf("dead    = %d\n", inConcrete.getDist().getDead());
	       printf("serious = %d\n", inConcrete.getDist().getSerious());
	       printf("slight  = %d\n", inConcrete.getDist().getSlight());
	       printf("unhurt  = %d\n", inConcrete.getDist().getUnhurt());
	       printf("outdoor\n");
	       printf("total   = %d\n", outdoor.getDist().getTotal());
	       printf("dead    = %d\n", outdoor.getDist().getDead());
	       printf("serious = %d\n", outdoor.getDist().getSerious());
	       printf("slight  = %d\n", outdoor.getDist().getSlight());
	       printf("unhurt  = %d\n", outdoor.getDist().getUnhurt());
#endif
	       //      setDegree(inWooden);
	       setDegreeAll();
	  }
		
     };
	
     class ObjPosition {
     private:
	  ObjNumInfo indoor;
	  ObjNumInfo outdoor;
     public:
	  ObjPosition() : indoor(), outdoor() {
	  }
	  ObjNumInfo& getIndoor() { return (indoor); }
	  ObjNumInfo& getOutdoor() { return (outdoor); }
     };
	
}

#endif
