
#include "common.hxx"
#include "basic.hxx"
#include "Config.hxx"
#include "Address.hxx"
#include "Input.hxx"
#include "Output.hxx"
#include "LongUDPSocket.hxx"
#include "RescueObjectPool.hxx"
#include "objects.hxx"
using namespace Rescue;

struct Client {
	Address m_address;
	S32 m_protocolVersion;
	S32 m_temporaryIdToKernel;
	S32 m_temporaryIdToClient;
	S32 m_agentType;
	S32 m_id;
	RealObject* m_controlledObject;
	Client() {
		m_protocolVersion = 0;
		m_temporaryIdToKernel = 0;
		m_temporaryIdToClient = 0;
		m_agentType = 0;
		m_id = 0;
		m_controlledObject = 0;
	}
};

int main2(int argc, char** argv);
int main(int argc, char** argv)
{
	try {
		return main2(argc, argv);
	} catch(std::exception& e) {
		fprintf(stderr, "%s\n", (const char*)e.what());
	} catch(...) {
		fprintf(stderr, "error...\n");
	}
	return 1;
}

void sendConnect(const Client* client, Output& output, LongUDPSocket& socket, const Address& kernel)
{
	output.clear();
	output.put(AK_CONNECT);
	Output::Cursor base = output.put(~(S32)0);
	output.put(client->m_temporaryIdToKernel);
	output.put((S32)2);			// version
	output.put(client->m_agentType);
	output.setSize(base);
	output.put(HEADER_NULL);
	socket.send(output, kernel, -1);
}
void sendAck(Id id, Output& output, LongUDPSocket& socket, const Address& kernel)
{
	output.clear();
	output.put(AK_ACKNOWLEDGE);
	Output::Cursor base = output.put(~(S32)0);
	output.put(id);
	output.setSize(base);
	output.put(HEADER_NULL);
	socket.send(output, kernel, -1);
}
void sendConnectOk(const Client* client, Output& output, ObjectPool& pool, LongUDPSocket& socket)
{
	output.clear();
	output.put(KA_CONNECT_OK);
	Output::Cursor base = output.put(~(S32)0);
	output.put(client->m_temporaryIdToClient);
	output.put(client->m_id);
	client->m_controlledObject->outputObject(output);
	if(client->m_protocolVersion == 0) {
		pool.output(output);
	}
	output.setSize(base);
	output.put(HEADER_NULL);
	socket.send(output, client->m_address);
}

void sendToAgent(S32 time, LongUDPSocket& socket, Output& output, Client* client, RescueObjectPool& pool, Config& config)
{
	const double squaredVision = (double)config.vision() * config.vision();
	output.clear();
	output.put(KA_SENSE);
	Output::Cursor base = output.put(~(S32)0);
	RealObject* self = client->m_controlledObject;
	output.put(self->id());
	output.put(time);
	// self
	self->outputObject(output);
	// map (苗)
	RescueObjectPool::Enumerator e = pool.objectsInRound(self->x(), self->y(), config.vision());
	Object* o;
	while(o = e.next(), o != 0)
		o->outputObject(output);
	// mapi΍Ёj
	Objects::const_iterator it2 = pool.firedObjects().begin();
	for(; it2 != pool.firedObjects().end(); it2++) {
		Building* b = dynamic_cast<Building*>(*it2);
		if(b != 0) {
			double x = (double)b->x() - self->x();
			double y = (double)b->y() - self->y();
			if(x * x + y * y > squaredVision)
				b->outputObject(output);
		}
	}
	output.put(TYPE_NULL);
	output.setSize(base);
	output.put(HEADER_NULL);
	socket.send(output, client->m_address, time);
}

int main2(int argc, char** argv)
{
	// This is very long function.  Don't read. :)
	
	const char* host = "localhost";
	Config config;
	
	// 
	int i;
	int count = 0;
	for(i=1; i<argc; i++) {
		if(argv[i][0] == '-') {
			if(strcmp(argv[i], "-file") == 0) {
				if(++i < argc) {
					config.readConfigFile(argv[i]);
				} else {
					fprintf(stderr, "error: '-file'\n");
				}
			} else {
				if(++i < argc) {
					if(!config.setValue(argv[i-1]+1, argv[i]))
						fprintf(stderr, "error: '%s'\n", argv[i]);
				} else {
					fprintf(stderr, "error: '%s'\n", argv[i-1]);
				}
			}
		} else {
			switch(count++) {
			case 0:
				host = argv[i];
				break;
			default:
				fprintf(stderr, "error: '%s'\n", argv[i]);
				break;
			}
		}
	}

	// \Pbg쐬
	LongUDPSocket socket(Address(INADDR_ANY, (S16)config.proxy_port(), true), config.textlogname_proxy());
	Input input;
	Output output;
	output.setWait(config.send_udp_wait());
	output.resetFrameSize(config.send_udp_size());
	Address kernel(host, config.port());
	RescueObjectPool pool;
	
	// kernel ֐ڑ
	printf("connecting...\n");
	output.clear();
	output.put(SK_CONNECT);
	Output::Cursor base = output.put(~(S32)0);
	output.put(0);
	output.setSize(base);
	output.put(HEADER_NULL);
	socket.send(output, kernel, -1);
	
	// ҂IuWFNgQ
	typedef std::map<Id, AutoPtr<Client> > IdToClient;
	IdToClient connectOkWaiters;
	IdToClient connectAckWaiters;
	IdToClient activeClients;
	Client* delayedAckSender = 0;
	int countConnectOkToDelayedAckSender = 0;
	S32 nextTemporalId = 1;
	
	// bZ[W[v
	S32 time = 0;
	int bufferSize = config.say_max_bytes();
	Byte* buffer = new Byte[bufferSize];
	std::vector<const LongUDPSocket*> sockets;
	sockets.push_back(&socket);
	for(;;) {
		try {
			if(countConnectOkToDelayedAckSender >= 5 || socket.wait(sockets, 5 * config.step()) <= 0) {
				countConnectOkToDelayedAckSender = 0;
				IdToClient::const_iterator it = connectOkWaiters.begin();
				for(; it != connectOkWaiters.end(); it++) {
					Client* client = it->second.get();
					sendConnect(client, output, socket, kernel);
				}
				it = connectAckWaiters.begin();
				for(; it != connectAckWaiters.end(); it++) {
					Client* client = it->second.get();
					sendConnectOk(client, output, pool, socket);
				}
			} else {
				socket.receive(input);
				for(;;) {
					Header header = input.get();
					if(header == HEADER_NULL)
						break;
					
					S32 size = input.get();
					Input::Cursor start = input.cursor();
					
					switch(header) {

					// vNVEJ[l̐ڑ
					case KS_CONNECT_ERROR:
						printf("Connection failed.\n");
						return 1;
					case KS_CONNECT_OK:
						printf("Connect Ok.\n");
						// ack Ԃ
						output.clear();
						output.put(SK_ACKNOWLEDGE);
						output.put(~(S32)0);
						output.put(HEADER_NULL);
						socket.send(output, kernel, -1);
						
						printf("Initializing...\n");
						kernel = socket.addressRecievedFrom();
						pool.restructure(0, input);
						pool.setUpMesh(config.mesh_size(), config.mesh_size());
						printf("Initialized.\n");
						break;
					
					// G[WFgEvNV̐ڑ
					case AK_CONNECT:
					{
						S32 temporaryId = input.get();
						S32 version = input.get();
						if(version != 0 && version != 1) {
							output.clear();
							output.put(KA_CONNECT_ERROR);
							Output::Cursor base = output.put(~(S32)0);
							output.put(temporaryId);
							output.putString("unknown verison for proxy");
							output.setSize(base);
							output.put(HEADER_NULL);
							socket.send(output, socket.addressRecievedFrom(), -1);
							break;
						}
						S32 agentType = input.get();
						
						AutoPtr<Client> client(new Client());
						client->m_protocolVersion = version;
						client->m_address = socket.addressRecievedFrom();
						client->m_temporaryIdToKernel = nextTemporalId++;
						client->m_temporaryIdToClient = temporaryId;
						client->m_agentType = agentType;
						
						connectOkWaiters[client->m_temporaryIdToKernel] = client;
						sendConnect(client.get(), output, socket, kernel);
					}
						break;
					case KA_CONNECT_OK:
					{
						S32 temporaryId = input.get();
						Id selfId = input.get();
						IdToClient::iterator it = connectOkWaiters.find(temporaryId);
						if(it != connectOkWaiters.end()) {
							AutoPtr<Client> client = it->second;
							client->m_id = selfId;
							client->m_controlledObject = dynamic_cast<RealObject*>(pool.get(selfId));
							connectOkWaiters.erase(it);
							
							connectAckWaiters[selfId] = client;
							sendConnectOk(client.get(), output, pool, socket);

							// ̃NCAgɂƂẮA߂Ă connectOk
							if(delayedAckSender != 0 && delayedAckSender->m_temporaryIdToKernel != temporaryId) {
								//  delayedAckSender Ƃ͈ႤNCAgɂĂ̂́؂ւ
								sendAck(delayedAckSender->m_id, output, socket, kernel);
							}
							delayedAckSender = client.get();
							countConnectOkToDelayedAckSender = 1;
						} else {
							//  connectOk MĂB
							if(delayedAckSender != 0 && delayedAckSender->m_temporaryIdToKernel == temporaryId) {
								// xĂ̂ɁA܂connectOk
								countConnectOkToDelayedAckSender++;
								if(connectOkWaiters.empty() && connectAckWaiters.empty()) {
									// ҂ؖ  xĂAck𑗐M
									sendAck(delayedAckSender->m_id, output, socket, kernel);
									delayedAckSender = 0;
									countConnectOkToDelayedAckSender = 1;
								}
							} else {
								// xĂȂii.e. Ack 𑗂ĵconnectOk܂đ
								sendAck(selfId, output, socket, kernel);
							}
						}
					}
						break;
					case KA_CONNECT_ERROR:
					{
						S32 temporaryId = input.get();
						std::string message = input.getString();
						
						IdToClient::iterator it = connectOkWaiters.find(temporaryId);
						if(it != connectOkWaiters.end()) {
							AutoPtr<Client> client = it->second;
							connectOkWaiters.erase(it);
							
							// relay it
							output.clear();
							output.put(KA_CONNECT_ERROR);
							Output::Cursor base = output.put(~(S32)0);
							output.put(client->m_temporaryIdToClient);
							output.putString(message.c_str());
							output.setSize(base);
							output.put(HEADER_NULL);
							socket.send(output, client->m_address, -1);
						}
					}
						break;
					case AK_ACKNOWLEDGE:
					{
						Id id = input.get();
						IdToClient::iterator it = connectAckWaiters.find(id);
						if(it != connectAckWaiters.end()) {
							AutoPtr<Client> client = it->second;
							connectAckWaiters.erase(it);
							
							activeClients[id] = client;
						}
					}
						break;

					// ʏ̏
					case KS_UPDATE:
					{
						time = input.get();
						pool.clearFiredObjects();
						pool.restructure(time, input);
						IdToClient::iterator it = activeClients.begin();
						for(; it !=  activeClients.end(); it++) {
							Client* client = it->second.get();
							sendToAgent(time+1, socket, output, client, pool, config);
						}
					}
						break;
					case KS_COMMANDS:
						break;
					// J[lG[WFg
					case KA_HEAR:
					{
						Id id = input.get();
						IdToClient::iterator it = activeClients.find(id);
						if(it != activeClients.end()) {
							Client* client = it->second.get();
							S32 sizeBound = (size - 4 + 3) & ~3;
							if(bufferSize < sizeBound) {
								delete[] buffer;
								bufferSize *= 2;
								buffer = new Byte[bufferSize];
							}
							input.get(sizeBound, buffer);
							// relay it
							output.clear();
							output.put(header);
							Output::Cursor base = output.put(~(S32)0);
							output.put(id);
							output.put(size-4, buffer);
							output.setSize(base);
							output.put(HEADER_NULL);
							socket.send(output, client->m_address, -1);
						}
					}
						break;
					
					// G[WFgJ[l
					default:
					{
						if(header < HEADER_COMMAND_MIN) {
							printf("Unknown header.  0x%lX\n", (long)header);
							break;
						}
						
						// R}h
						S32 sizeBound = (size + 3) & ~3;
						if(bufferSize < sizeBound) {
							delete[] buffer;
							bufferSize *= 2;
							buffer = new Byte[bufferSize];
						}
						input.get(sizeBound, buffer);
						// relay it
						output.clear();
						output.put(header);
						Output::Cursor base = output.put(~(S32)0);
						output.put(size, buffer);
						output.setSize(base);
						output.put(HEADER_NULL);
						socket.send(output, kernel, -1);
					}
						break;
					}
	
					input.setCursor(start);
					input.skip(size);
				}
			}
		} catch(std::exception& e) {
			fprintf(stderr, "%s\n", (const char*)e.what());
		} catch(...) {
			fprintf(stderr, "error...\n");
		}
	}
	
	delete[] buffer;
	return 0;
}

